﻿#include "DOshooter.h"

LPDIRECT3D9 pD3d;             //pointer to Direct3d object.
LPDIRECT3DDEVICE9 pDevice;    //pointer to Direct3d device object.

LPDIRECTINPUT8 pDinput;            //pointer of DirectInput object.
LPDIRECTINPUTDEVICE8 pKeyDevice;   //pointer of DirectInput device object.
LPD3DXFONT m_pFont;                //something about fonts.

#define WIDTH 1280
#define HEIGHT 720

HWND hWnd;              //handle.
WNDCLASSEX wc = { 0 };  //windows class.


OBJ ship = {   //my ship.
	{ 640, 600, 0 },  //cordinates.
	"img/ship.png",
	TRUE,
	{16,10,46,64},
	NULL,NULL
};


LRESULT CALLBACK WinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);  //window procedure.
HRESULT D3dinit(HWND hWnd);                                                       //DirectX9 initializer.
HRESULT DInpinit(HWND hWind, HINSTANCE hInstance);                                //DirectInput initializer.
void AppProc(void);                                                               //ship procedure.


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {

	MSG msg = {0};  //struct which holds event messages.

	/* set window class */
	wc.cbSize = sizeof(WNDCLASSEX);                      //set size.
	wc.style = CS_HREDRAW | CS_VREDRAW;                  //re-draw window when vertical and horizontal window size is changed.
	wc.lpfnWndProc = WinProc;                            //tell window procedure.
	wc.hInstance = hInstance;                            //set instance handle.
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);            //set cursor information.
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);      //set background color.
	wc.lpszClassName = "DOShooterMain";                  //name of window class.

	RegisterClassEx(&wc);  //register window class.

	hWnd = CreateWindowEx(	//create window.
		WS_EX_APPWINDOW,                   //focus on taskbar when window is visualized.
		"DOShooterMain",                   //select window class
		"DOshooter - pre-Alpha",           //window title.
		WS_EX_LEFT | WS_OVERLAPPEDWINDOW ^ WS_MAXIMIZEBOX ^ WS_THICKFRAME | WS_VISIBLE,   //disable size change except minimizing, visualize window when created.
		300, 300,                          //initial XY coordinates of window.
		WIDTH, HEIGHT,                     //window width and height.
		NULL, NULL,                        //no parent window, no menu.
		hInstance, NULL
	);

	if (D3dinit(hWnd) != S_OK)	return 0;              //initialize DirectX.
	if (DInpinit(hWnd, hInstance) != S_OK)	return 0;  //initialize DirectInput.

	while (msg.message != WM_QUIT) { //main loop
		
		if ( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) ) {  //get message if exists.

			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else{                                              //if there's no mesage,
			
			ClearS();      //clear screen.
			AmmoProc();    //draw ammo.
			AppProc();     //get keyboard input and draw ship.

			IDirect3DDevice9_Present(pDevice, NULL, NULL, NULL, NULL);  //flip bitween Backbuffer and Frontbuffer.
		}
	}

	UnregisterClass("DOShooterMain", hInstance);
	return msg.wParam;
}


/*
  Window procedure.
*/

LRESULT CALLBACK WinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

	switch (message) {

	case WM_KEYDOWN:           //if key pressed,
		
		if (wParam == VK_ESCAPE) {  //if Escape key pushed,
			PostQuitMessage(0);     //quit process.
		}

		return 0;

	case WM_DESTROY:  //quit process.

		PostQuitMessage(0);
		return 0;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
}


/*
  Function which Draws sprite of the ship.
*/

void AppProc(void) {

	HRESULT hr = IDirectInputDevice_Acquire(pKeyDevice);   //get device.

	if ( (hr == DI_OK) || (hr == S_FALSE) ) {    //if got the device,
		
		BYTE diks[256];                           //keyboard state container.
		IDirectInputDevice_GetDeviceState(pKeyDevice, sizeof(diks), &diks);  //acquire keyboard state.
		
		if (diks[DIK_UP] & 0x80) {		//if up key pressed,
			
			if(ship.cords.y >= 4)	ship.cords.y -= 4;  //move up.
		}
		if (diks[DIK_DOWN] & 0x80) {	//if down key pressed,
			
			if(ship.cords.y <= (716 - 64) )	ship.cords.y += 4;	//move down.
		}

		if (diks[DIK_LEFT] & 0x80) {    //if left key pessed,
			
			if(ship.cords.x >= 4)	ship.cords.x -= 4;	//move left.
		}
		
		if (diks[DIK_RIGHT] & 0x80) {	//if right key pessed,
			if(ship.cords.x <= (1276 - 64) )	ship.cords.x += 4;	//move right.
		}
	}


	if (SUCCEEDED(IDirect3DDevice9_BeginScene(pDevice))) {

		ship.pSprite->Begin(D3DXSPRITE_ALPHABLEND);
		ship.pSprite->Draw(   //draw sprite.
			ship.pTexture,      //texture.
			NULL,		  	    //
			NULL,			    //
			&ship.cords,        //
			0xFFFFFFFF          //do not chage looks of the picture.
		);

		ship.pSprite -> End();
		IDirect3DDevice9_EndScene(pDevice);
	}
}


/*
  Directx9 initializer. 
  Creates DirectX object, device object, and ship's texture and sprite for now.
*/

HRESULT D3dinit(HWND hWind) {

	pD3d = Direct3DCreate9(D3D_SDK_VERSION);  //create DirectX object.
	
	/* create device object. */
	D3DPRESENT_PARAMETERS d3dpp = {0};              //initialize struct.
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;        //unknown.
	d3dpp.BackBufferCount = 1;                      //use one backbaffer.
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;       //something.
	d3dpp.Windowed = TRUE;                          //window mode.

	if (0 > IDirect3D9_CreateDevice(       //store device object. use hardware process and hardware vertex process.
			pD3d,
			D3DADAPTER_DEFAULT,
			D3DDEVTYPE_HAL,            //use hardware to inprove performance(?) faster.
			hWnd,
			D3DCREATE_HARDWARE_VERTEXPROCESSING,  //use hardware to process vertex. faster.
			&d3dpp,
			&pDevice
	)) {  //if failed,

		if (0 > IDirect3D9_CreateDevice(       //store device object. use hardware process and software vertex process.
				pD3d,
				D3DADAPTER_DEFAULT,
				D3DDEVTYPE_HAL,            //use hardware to inprove performance(?) faster.
				hWnd,
				D3DCREATE_SOFTWARE_VERTEXPROCESSING,  //use software to process vertex. faster.
				&d3dpp,
				&pDevice
		)) {  //if failed,

			if (0 > IDirect3D9_CreateDevice(       //store device object. use software process and hardware vertex process.
					pD3d,
					D3DADAPTER_DEFAULT,
					D3DDEVTYPE_REF,            //use software to inprove performance(?) faster.
					hWnd,
					D3DCREATE_HARDWARE_VERTEXPROCESSING,  //use hardware to process vertex. faster.
					&d3dpp,
					&pDevice
			)) {  //if failed

				if (0 > IDirect3D9_CreateDevice(       //store device object. use software process and software vertex process.
						pD3d,
						D3DADAPTER_DEFAULT,
						D3DDEVTYPE_REF,            //use software to inprove performance(?) faster.
						hWnd,
						D3DCREATE_SOFTWARE_VERTEXPROCESSING,  //use software to process vertex. faster.
						&d3dpp,
						&pDevice
				)) {
					MessageBox(0, "Unable to create DIRECT3D device. LOL", NULL, MB_OK);
					return E_FAIL;
				}
			}
		}
	}

		/* create texture object */
	D3DXCreateTextureFromFileEx(
		pDevice,
		ship.img,
		D3DX_DEFAULT,      //use image width.
		D3DX_DEFAULT,      //use image height.
		D3DX_DEFAULT,      //mipmap level.
		0,                 //???
		D3DFMT_UNKNOWN,    //file format
		D3DPOOL_DEFAULT,   //???
		D3DX_FILTER_NONE,  //no filters.
		D3DX_DEFAULT,      //no mipfilters.
		0xFF000000,        //make specific color to be invisible, ARGB. in this case, black will be invisible.
		NULL,
		NULL,
		&ship.pTexture
	);

	D3DXCreateSprite(pDevice, &ship.pSprite);  //create sprite.

	AmmoInit();  //initialize ammo.

	return S_OK;
}


/*
  DirectInput initializer.
*/

HRESULT DInpinit(HWND hWind, HINSTANCE hInstance) {

	HRESULT hr;  //error container.

	hr = DirectInput8Create(hInstance, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&pDinput, NULL);  //create DirectInput object.
	if (FAILED(hr))	return hr;
	
	hr = IDirectInput8_CreateDevice(pDinput, GUID_SysKeyboard, &pKeyDevice, NULL);  //create DirectInput device object
	if (FAILED(hr))	return hr;

	hr = IDirectInputDevice_SetDataFormat(pKeyDevice, &c_dfDIKeyboard);                //Set the device to keyboard.
	if (FAILED(hr))	return hr;


	hr = IDirectInputDevice_SetCooperativeLevel(pKeyDevice, hWnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND); //set cooperative level.
	if (FAILED(hr))	return hr;

	IDirectInputDevice_Acquire(pKeyDevice);  //get device.
	
	return S_OK;
}